@@ -15,6 +15,14 @@
 /* Define if you have _GNU_SOURCE getpt() */
 #undef HAVE_GETPT
 
+/* Define if you have a broken version of the `vsnprintf' function */
+#undef HAVE_SNPRINTF_UNTERM
+#undef HAVE_SNPRINTF_BUG
+#undef HAVE_VSNPRINTF_UNTERM
+#undef HAVE_VSNPRINTF_BUG
+
+#undef HAVE_BROKEN_VSNPRINTF
+
 /* Define possible pty types */
 #undef PTYS_ARE_NUMERIC
 #undef PTYS_ARE_PTMX
