@@ -1,5 +1,22 @@
 
 /* changes:
+ * 1.15:
+ *   - splint hsftp.c and piper.c
+ *   - check if (v)snprintf works correctly, and use if yes
+ *   - make drop-root the default
+ *   - use initgroups() if available
+ *
+ * 1.14:
+ *   - fix format string security bugs in hsftp.c and piper.c 
+ *     (detected by Ulf Hrnhammar)
+ *     might be exploitable by a malicious server
+ *
+ * 1.13:
+ *   - null terminate lin_str (strncpy will not do that)
+ *   - null terminate pwd     (strncpy will not do that)
+ *   - limit length of username, hostname arguments 
+ *     (buffer overflow detected by Xavier Brouckaert)
+ *
  * 1.12:
  *   - allow y/n for Y/N
  *   - fix command line parsing
@@ -65,6 +82,7 @@
 #include <string.h>
 #include <errno.h>
 #include <pwd.h>
+#include <grp.h>
 #include <unistd.h>
 #include <sys/wait.h>
 #include <signal.h>
@@ -98,6 +116,13 @@
 #define TRUE (!FALSE)
 #endif
 
+#ifndef HAVE_SNPRINTF
+#define HAVE_SNPRINTF_BUG
+#endif
+#ifndef HAVE_VSNPRINTF
+#define HAVE_VSNPRINTF_BUG
+#endif
+
 /* There are at most 4 args of size MEDBUF plus the format for sprintf,
  * thus we allocate MAXBUF = (5*MEDBUF) for the command string.
  */
@@ -119,7 +144,7 @@ int opt_tarflag = TRUE;
 int is_suid_root = FALSE;
 int no_mlock     = TRUE;
 
-int child_pid = 0;
+pid_t child_pid = 0;
 unsigned int port = 22;  /* SSH port */
 
 char * lin_str = NULL;
@@ -162,24 +187,24 @@ enum mode_color  {
 
 static struct len_str the_colors[] =
 {
-  { sizeof ("\033[") -1, ("\033[") },       /* start sequence */
+  { (int) sizeof ("\033[") -1, ("\033[") },       /* start sequence */
 
-  { sizeof ("32") -1,    ("32")    },       /* green   */
-  { sizeof ("31") -1,    ("31")    },       /* red     */
-  { sizeof ("36") -1,    ("36")    },       /* blue    */
-  { sizeof ("34") -1,    ("34")    },       /* cyan    */
-  { sizeof ("35") -1,    ("35")    },       /* magenta */
-  { sizeof ("01;31") -1, ("01;31") },       /* red     */
+  { (int) sizeof ("32") -1,    ("32")    },       /* green   */
+  { (int) sizeof ("31") -1,    ("31")    },       /* red     */
+  { (int) sizeof ("36") -1,    ("36")    },       /* blue    */
+  { (int) sizeof ("34") -1,    ("34")    },       /* cyan    */
+  { (int) sizeof ("35") -1,    ("35")    },       /* magenta */
+  { (int) sizeof ("01;31") -1, ("01;31") },       /* red     */
 
-  { sizeof ("0") -1,     ("0")     },       /* Default      */
-  { sizeof ("01;32") -1, ("01;32") },       /* directory    */
-  { sizeof ("01;36") -1, ("01;36") },       /* symlink      */
-  { sizeof ("01;31") -1, ("01;31") },       /* fifo         */
-  { sizeof ("01;33") -1, ("01;33") },       /* socket       */
-  { sizeof ("44;37") -1, ("44;37") },       /* device       */
-  { sizeof ("01;35") -1, ("01;35") },       /* exec         */
+  { (int) sizeof ("0") -1,     ("0")     },       /* Default      */
+  { (int) sizeof ("01;32") -1, ("01;32") },       /* directory    */
+  { (int) sizeof ("01;36") -1, ("01;36") },       /* symlink      */
+  { (int) sizeof ("01;31") -1, ("01;31") },       /* fifo         */
+  { (int) sizeof ("01;33") -1, ("01;33") },       /* socket       */
+  { (int) sizeof ("44;37") -1, ("44;37") },       /* device       */
+  { (int) sizeof ("01;35") -1, ("01;35") },       /* exec         */
 
-  { sizeof ("m\0") -1,   ("m\0")   },       /* end   sequence */
+  { (int) sizeof ("m\0") -1,   ("m\0")   },       /* end   sequence */
 };
 
 
@@ -238,7 +263,7 @@ static void printcol (FILE * stream, char * str, char * c_mode,
 		      int start, int end)
 {
   register int i;
-  int len = strlen(str);
+  int len = (int) strlen(str);
 
   if (!opt_color)
     {
@@ -269,7 +294,7 @@ static void sigHandler (int mysignal)
       kill (child_pid, SIGTERM);
       sleep (1);
       kill (child_pid, SIGKILL);
-      waitpid (-1, NULL, WNOHANG|WUNTRACED);
+      (void) waitpid (-1, NULL, WNOHANG|WUNTRACED);
       child_pid = 0;
       return;
     }
@@ -302,7 +327,7 @@ void * my_malloc (size_t size)
 static char * my_strdup (const char * str)
 {
   char * newstr;
-  int  len = strlen(str);
+  size_t  len = strlen(str);
 
   newstr = my_malloc (len + 1);
   strcpy (newstr, str);        /* known to fit */
@@ -311,7 +336,7 @@ static char * my_strdup (const char * str)
 
 static char * my_strconcat (const char * arg1, ...)
 {
-  int     length;
+  size_t    length;
   char    * s;
   char    * strnew;
   va_list vl;
@@ -356,11 +381,18 @@ void print_dbg (const char *fmt, ...)
 
    if (!opt_debug)
      return;
+   if (!err_str)
+     return;
 
    va_start(vl,fmt);
-   vsprintf(err_str, fmt, vl);
+#ifdef HAVE_VSNPRINTF_BUG
+   vsprintf  (err_str, fmt, vl);
+#else
+   vsnprintf (err_str, MEDBUF+MAXBUF, fmt, vl);
+   err_str[MEDBUF+MAXBUF-1] = '\0';
+#endif
    va_end(vl);
-   fprintf (stderr, err_str);
+   fprintf (stderr, "%s", err_str);
    fputs("\n", stderr);
    return;
 }
@@ -499,6 +531,7 @@ static void dirz (char *path)
 void privileges(int mode)
 {
   static int i = 0;
+  struct passwd *passit;
 
   if (i == 0)
     {
@@ -510,6 +543,12 @@ void privileges(int mode)
 	  fprintf(stderr, "**PANIC** Could not drop privileges !!!\n");
 	  _exit(EXIT_FAILURE);
 	}
+#ifdef HAVE_INITGROUPS
+      if (NULL != (passit = getpwuid(getuid())))
+	{
+	  initgroups(passit->pw_name, passit->pw_gid);
+	}
+#endif
     }
 
   return;
@@ -586,8 +625,14 @@ static void cmd_cwd ()
 {
   /* 'echo $PWD' -> pwd suggested by Tomi Ollila
    */
-  sprintf (cmd_str, "ssh -p %d -x -l %s %s 'pwd'", 
+#ifdef HAVE_SNPRINTF_BUG
+  sprintf (cmd_str, "ssh -p %u -x -l %s %s 'pwd'", 
 	   port, usr_str, hst_str);
+#else
+  snprintf (cmd_str, MAXBUF, "ssh -p %u -x -l %s %s 'pwd'", 
+	   port, usr_str, hst_str);
+  cmd_str[MAXBUF-1] = '\0';
+#endif
   print_dbg("Command: %s", cmd_str);
   piper (cmd_str, 0);
   strncpy (cwd_str, hom_str, MEDBUF);
@@ -598,7 +643,7 @@ static void cmd_cwd ()
 static void cmd_cd (char * str)
 {
   char * p;
-  int    len;
+  size_t len;
 
   len = strlen(str);
   if (str[len-1] == '\n') str[len-1] = '\0';
@@ -633,7 +678,7 @@ static void cmd_cd (char * str)
 
 static void cmd_dir (char * str)
 {
-  int    len;
+  size_t len;
   char * p;
 
   len = strlen(str);
@@ -643,12 +688,22 @@ static void cmd_dir (char * str)
 
   /* no argument
    */
+#ifdef HAVE_SNPRINTF_BUG
   if (*p == '\n' || *p == '\0')
-      sprintf (cmd_str, "ssh -p %d -x -l %s %s 'ls %s'", 
+      sprintf (cmd_str, "ssh -p %u -x -l %s %s 'ls %s'", 
 	       port, usr_str, hst_str, cwd_str);
   else
-    sprintf (cmd_str, "ssh -p %d -x -l %s %s 'cd %s && ls %s'",
+    sprintf (cmd_str, "ssh -p %u -x -l %s %s 'cd %s && ls %s'",
 	     port, usr_str, hst_str, cwd_str, p);
+#else
+  if (*p == '\n' || *p == '\0')
+      snprintf (cmd_str, MAXBUF, "ssh -p %u -x -l %s %s 'ls %s'", 
+	       port, usr_str, hst_str, cwd_str);
+  else
+    snprintf (cmd_str, MAXBUF, "ssh -p %u -x -l %s %s 'cd %s && ls %s'",
+	     port, usr_str, hst_str, cwd_str, p);
+  cmd_str[MAXBUF-1] = '\0';
+#endif
 
   print_dbg("Command: %s", cmd_str);
   piper (cmd_str, 1);
@@ -813,7 +868,7 @@ int is_white_space (char c)
  */
 void stripwhite (char * string)
 {
-  register int i = 0;
+  register size_t i = 0;
 
   while (is_white_space (string[i]))
     i++;
@@ -1067,6 +1122,11 @@ int main (int argc, char * argv[])
 	    usage(FALSE);
 	  if (argv[0][0] == '-')
 	    usage(FALSE);
+	  if (64 < strlen(argv[0]))
+	    {
+	      fprintf(stderr, "** Error ** Username too long\n");
+	      usage(FALSE);
+	    }
 	  usr_str = my_strdup (argv[0]);
 	  break;
 	case 'p':
@@ -1090,7 +1150,14 @@ int main (int argc, char * argv[])
       --argc; ++argv;
     }
   if (argc == 1)
-    hst_str = my_strdup (argv[0]);
+    {
+      if (128 < strlen(argv[0]))
+	{
+	  fprintf(stderr, "** Error ** Hostname too long\n");
+	  usage(FALSE);
+	}
+      hst_str = my_strdup (argv[0]);
+    }
   else
     usage(FALSE);
 
@@ -1162,10 +1229,12 @@ int main (int argc, char * argv[])
 	add_history (line_read);
 
       strncpy (lin_str, line_read, MEDBUF-1);
+      lin_str[MEDBUF-1] = '\0';                 /* fix: null terminate */
       free(line_read);
       line_read = NULL;
 #else
       printf (prompt_str);
+      /* fgets() reads at most MEDBUF-2 chars and null terminates */
       if (NULL == (fgets  (lin_str, MEDBUF-1, stdin)))
 	  break;
       stripwhite (lin_str);
@@ -1192,13 +1261,27 @@ int main (int argc, char * argv[])
 	   * (actually, just a guess after stracing scp).
 	   */
 #ifdef OPEN_SSH
+#ifdef HAVE_SNPRINTF_BUG
 	  sprintf(cmd_str, 
-		  "scp -P %d -r -q %s %s@%s:%s",
+		  "scp -P %u -r -q %s %s@%s:%s",
 		  port, p, usr_str, hst_str, cwd_str);
 #else
-	  sprintf(cmd_str, 
-		  "scp -P %d -r %s %s@%s:%s",
+	  snprintf(cmd_str, MAXBUF,
+		  "scp -P %u -r -q %s %s@%s:%s",
 		  port, p, usr_str, hst_str, cwd_str);
+	  cmd_str[MAXBUF-1] = '\0';
+#endif
+#else
+#ifdef HAVE_SNPRINTF_BUG
+	  sprintf(cmd_str, 
+		  "scp -P %u -r %s %s@%s:%s",
+		  port, p, usr_str, hst_str, cwd_str);
+#else
+	  snprintf(cmd_str, MAXBUF,
+		  "scp -P %u -r %s %s@%s:%s",
+		  port, p, usr_str, hst_str, cwd_str);
+	  cmd_str[MAXBUF-1] = '\0';
+#endif
 #endif
 	  print_dbg("Command: %s", cmd_str);
 	  piper (cmd_str, 2);
@@ -1234,9 +1317,16 @@ int main (int argc, char * argv[])
 	  if (*p == '\0')
 	    continue;
 
+#ifdef HAVE_SNPRINTF_BUG
 	  sprintf(cmd_str, 
-		  "ssh -p %d -x -l %s %s 'cd %s && tar cfb - 20 %s ' | tar xvBfb - 20",
+		  "ssh -p %u -x -l %s %s 'cd %s && tar cfb - 20 %s ' | tar xvBfb - 20",
 		  port, usr_str, hst_str, cwd_str, p);
+#else
+	  snprintf(cmd_str, MAXBUF,
+		  "ssh -p %u -x -l %s %s 'cd %s && tar cfb - 20 %s ' | tar xvBfb - 20",
+		  port, usr_str, hst_str, cwd_str, p);
+	  cmd_str[MAXBUF-1] = '\0';
+#endif
 	  print_dbg("Command: %s", cmd_str);
 	  piper (cmd_str, 2);
 	  printf ("Finished\n");
@@ -1253,9 +1343,17 @@ int main (int argc, char * argv[])
 	  if (*p == '\0')
 	    continue;
 
+#ifdef HAVE_SNPRINTF_BUG
 	  sprintf(cmd_str, 
-		  "ssh -p %d -x -l %s %s 'cd %s && rm %s '",
+		  "ssh -p %u -x -l %s %s 'cd %s && rm %s '",
 		  port, usr_str, hst_str, cwd_str, p);
+#else
+	  snprintf(cmd_str, MAXBUF, 
+		  "ssh -p %u -x -l %s %s 'cd %s && rm %s '",
+		  port, usr_str, hst_str, cwd_str, p);
+	  cmd_str[MAXBUF-1] = '\0';
+#endif
+
 	  print_dbg("Command: %s", cmd_str);
 	  piper (cmd_str, 2);
 	  continue;
@@ -1267,9 +1365,16 @@ int main (int argc, char * argv[])
 	  if (lin_str[strlen(lin_str)-1] == '\n')
 	    lin_str[strlen(lin_str)-1] = '\0';
 	  
+#ifdef HAVE_SNPRINTF_BUG
 	  sprintf(cmd_str, 
-		  "ssh -p %d -x -l %s %s 'cd %s && %s '",
+		  "ssh -p %u -x -l %s %s 'cd %s && %s '",
 		  port, usr_str, hst_str, cwd_str, lin_str);
+#else
+	  snprintf(cmd_str, MAXBUF,
+		  "ssh -p %u -x -l %s %s 'cd %s && %s '",
+		  port, usr_str, hst_str, cwd_str, lin_str);
+	  cmd_str[MAXBUF-1] = '\0';
+#endif
 	  print_dbg("Command: %s", cmd_str);
 	  piper (cmd_str, 2);
 	  continue;
@@ -1280,9 +1385,16 @@ int main (int argc, char * argv[])
 	  if (lin_str[strlen(lin_str)-1] == '\n')
 	    lin_str[strlen(lin_str)-1] = '\0';
 	  
+#ifdef HAVE_SNPRINTF_BUG
 	  sprintf(cmd_str, 
-		  "ssh -p %d -x -l %s %s 'cd %s && %s '",
+		  "ssh -p %u -x -l %s %s 'cd %s && %s '",
 		  port, usr_str, hst_str, cwd_str, &lin_str[4]);
+#else
+	  snprintf(cmd_str, MAXBUF,
+		  "ssh -p %u -x -l %s %s 'cd %s && %s '",
+		  port, usr_str, hst_str, cwd_str, &lin_str[4]);
+	  cmd_str[MAXBUF-1] = '\0';
+#endif
 	  print_dbg("Command: %s", cmd_str);
 	  piper (cmd_str, 2);
 	  continue;
@@ -1305,7 +1417,12 @@ int main (int argc, char * argv[])
 
       if (0 == strncmp (lin_str, "ldir", 4))
 	{
+#ifdef HAVE_SNPRINTF_BUG
 	  sprintf (cmd_str, "ls %s", &lin_str[4]);
+#else
+	  snprintf (cmd_str, MAXBUF, "ls %s", &lin_str[4]);
+	  cmd_str[MAXBUF-1] = '\0';
+#endif
 	  system (cmd_str); /* LOCAL CALL */
 	  continue;
 	}
@@ -1319,7 +1436,8 @@ int main (int argc, char * argv[])
 	  if (*p == '\0')
 	    {
 	      pwent = getpwuid (geteuid());
-	      strncpy (pwd, pwent->pw_dir, MEDBUF);
+	      strncpy (pwd, pwent->pw_dir, MEDBUF-1); /* fix: null terminate */
+	      pwd[MEDBUF-1] = '\0';
 	    }
 	  else if (*p == '/')
 	    {
