@@ -50,6 +50,10 @@
 #define RESTORE 3
 #endif
 
+#ifndef O_NDELAY
+#define O_NDELAY O_NONBLOCK
+#endif
+
 extern char password[128];
 extern char passphrase[128];
 extern char * hom_str;
@@ -58,8 +62,8 @@ extern void get_pw();
 extern void get_pp();
 extern void privileges(int mode);
 
-extern int child_pid;
-extern int opt_tarflag;
+extern pid_t child_pid;
+extern int   opt_tarflag;
 
 static char    *ttydev = NULL;	/* pty/tty name */
 static struct stat ttyfd_stat;	/* original status of the tty we will use */
@@ -81,7 +85,7 @@ static short    changettyowner = 1;
  * master and slave parts
  */
 /* INTPROTO */
-int
+static int
 get_pty(void)
 {
     int             fd;
@@ -101,10 +105,12 @@ get_pty(void)
 #if defined(HAVE_GRANTPT) && defined(HAVE_UNLOCKPT)
 # if defined(PTYS_ARE_GETPT) || defined(PTYS_ARE_PTMX)
     {
-	extern char    *ptsname();
+        /*@i@*/ extern char    *ptsname();
 
 #  ifdef PTYS_ARE_GETPT
+	/*@-unrecog@*/
 	if ((fd = getpt()) >= 0) {
+	  /*@+unrecog@*/
 #  else
 	if ((fd = open("/dev/ptmx", O_RDWR)) >= 0) {
 #  endif
@@ -162,7 +168,7 @@ get_pty(void)
 #endif
 #ifdef PTYS_ARE_SEARCHED
     {
-	int             len;
+	size_t          len;
 	const char     *c1, *c2;
 	char            pty_name[] = "/dev/pty??";
 	char            tty_name[] = "/dev/tty??";
@@ -206,7 +212,7 @@ get_pty(void)
  * need to re-open the slave tty.
  */
 /* INTPROTO */
-int
+static int
 get_tty(void)
 {
     int             fd, i;
@@ -309,7 +315,7 @@ get_tty(void)
 
 #ifdef ultrix
     if ((fd = open("/dev/tty", O_RDONLY)) >= 0) {
-	ioctl(fd, TIOCNOTTY, 0);
+      ioctl(fd, /*@i@*/ TIOCNOTTY, 0);
 	close(fd);
     } else {
 	pid = setpgrp(0, 0);
@@ -320,14 +326,14 @@ get_tty(void)
 /* no error, we could run with no tty to begin with */
 #else				/* ultrix */
 # ifdef TIOCSCTTY
-    ioctl(0, TIOCSCTTY, 0);
+    ioctl(0, /*@i@*/ TIOCSCTTY, 0);
 # endif
 
 /* set process group */
 # if defined (_POSIX_VERSION) || defined (__svr4__)
     tcsetpgrp(0, pid);
 # elif defined (TIOCSPGRP)
-    ioctl(0, TIOCSPGRP, &pid);
+    ioctl(0, /*@i@*/ TIOCSPGRP, &pid);
 # endif
 
 /* svr4 problems: reports no tty, no job control */
@@ -342,101 +348,7 @@ get_tty(void)
 
 
 
-#if 0
-
-/* the following two subroutines are from 'Linux Application Development'
- * by Michael K. Johnson and Erik W. Troan (ptypair.c), 
- * and have been placed in the
- * public domain by their authors.
- */
-  
-/* get_master_pty() takes a double-indirect character pointer in which
- * to put a slave name, and returns an integer file descriptor.
- * If it returns < 0, an error has occurred.
- * Otherwise, it has returned the master pty file descriptor, and fills
- * in *name with the name of the corresponding slave pty.
- * Once the slave pty has been opened, you are responsible to free *name.
- */
-
-int get_master_pty(char **name) { 
-   int i, j = 0;
-   /* default to returning error */
-   int master = -1;
-
-   /* create a dummy name to fill in */
-   *name = strdup("/dev/ptyXX");
-
-   /* search for an unused pty */
-   for (i=0; i<16 && master <= 0; i++) {
-      for (j=0; j<16 && master <= 0; j++) {
-         (*name)[8] = "pqrstuvwxyzPQRST"[i];
-         (*name)[9] = "0123456789abcdef"[j];
-         /* open the master pty */
-         if ((master = open(*name, O_RDWR)) < 0) {
-            if (errno == ENOENT) {
-               /* we are out of pty devices */
-               free (*name);
-               return (master);
-            }
-         }
-      }
-   }
-   if ((master < 0) && (i == 16) && (j == 16)) {
-      /* must have tried every pty unsuccessfully */
-      free (*name);
-      return (master);
-   }
-
-   /* By substituting a letter, we change the master pty
-    * name into the slave pty name.
-    */
-   (*name)[5] = 't';
-
-   fcntl(master, F_SETFL, O_NDELAY);
-   return (master);
-}
-
-/* get_slave_pty() returns an integer file descriptor.
- * If it returns < 0, an error has occurred.
- * Otherwise, it has returned the slave file descriptor.
- */
-
-int get_slave_pty(char *name) { 
-   struct group *gptr;
-   gid_t gid;
-   int slave = -1;
-
-   /* chown/chmod the corresponding pty, if possible.
-    * This will only work if the process has root permissions.
-    * Alternatively, write and exec a small setuid program that
-    * does just this.
-    */
-   if ((gptr = getgrnam("tty")) != 0) {
-      gid = gptr->gr_gid;
-   } else {
-      /* if the tty group does not exist, don't change the
-       * group on the slave pty, only the owner
-       */
-      gid = -1;
-   }
-
-   stat (name, &ttyfd_stat);
-   
-   if (0 >= (slave = open(name, O_RDWR)))
-     {
-       /* 
-	  -- fails if no root privileges --
-	*/
-       privileges (RESTORE);
-       chown(name, getuid(), gid);
-       chmod(name, S_IRUSR|S_IWUSR);
-       privileges (IGNORE);
-     }
-   return (slave);
-}
-#endif
-
-void restore_tty (char * slave)
+static void restore_tty (char * slave)
 {
   /* fail silently if unprivileged
    */
@@ -456,7 +368,7 @@ static int errflag = 0;
  * from /dev/tty, and does not fallback on stdin.
  * Even more braindead: OpenSSH **PRINTS** the password prompt to /dev/tty !!
  */
-char * fdgets (char * buf, int size, int fd, int fderr
+static char * fdgets (char * buf, int size, int fd, int fderr
 #ifdef OPEN_SSH
 	       , int fdmaster
 #endif
@@ -469,7 +381,7 @@ char * fdgets (char * buf, int size, int fd, int fderr
   char             *  p;
 
   int                countbytes;
-  int                count;
+  ssize_t            count;
 
   int                select_now;
   int                num_sel;
@@ -689,15 +601,16 @@ char * fdgets (char * buf, int size, int fd, int fderr
 
 
 
-int user_handler(int mode, int master, int input_pipe[], int output_pipe[], 
-		  int inerr_pipe[])
+static int user_handler(int mode, int master, 
+			int input_pipe[], int output_pipe[], 
+			int inerr_pipe[])
 {
 
-  int rc;         /* return values of functions. */
-  char line[256];
-  char * p;
-  char   c;
-  int  i = 0;
+  ssize_t rc;         /* return values of functions. */
+  char    line[256];
+  char *  p;
+  char    c;
+  int     i = 0;
 
 
   /* first, close unnecessary file descriptors 
@@ -907,12 +820,12 @@ int user_handler(int mode, int master, int input_pipe[], int output_pipe[],
 	  p = &line[strlen(line)-1];
 	  while (*p) { if (*p == '\n') *p = '\0'; --p; }
 	  if (line[strlen(line)-1] == ':')
-	    { printf("\n\n"); printf(line); printf("\n"); }
+	    { printf("\n\n%s\n", line); }
 	  else if (strlen(line) > 40)
-	    { printf(line); printf("\n"); }
+	    { printf("%s\n", line); }
 	  else
 	    { 
-	      printf(line); 
+	      printf("%s", line); 
 	      if (strlen(line) < 8) 
 		printf("\t\t\t");
 	      else if (strlen(line) < 16)
@@ -946,8 +859,8 @@ int user_handler(int mode, int master, int input_pipe[], int output_pipe[],
 }
 
 /* now comes the function executed by the translator process. */
-void translator(char * command, int input_pipe[], int output_pipe[], 
-		int outerr_pipe[])
+static void translator(char * command, int input_pipe[], int output_pipe[], 
+		       int outerr_pipe[])
 {
 
 #ifdef DEBUG_P
@@ -996,8 +909,8 @@ int piper (char * command, int mode)
     int translator_to_user[2];
     int err_to_user[2];
 
-    int pid;       /* pid of child process, or 0 in parent thread. */
-    int rc;        
+    pid_t pid;       /* pid of child process, or 0 in parent thread. */
+    int   rc;        
 
     if (command == NULL)
       {
@@ -1113,7 +1026,7 @@ int piper (char * command, int mode)
 
     /* just to be sure
      */
-    waitpid (-1, NULL, WNOHANG|WUNTRACED);
+    (void) waitpid (-1, NULL, WNOHANG|WUNTRACED);
     child_pid = 0;
 
     close(master);
